
/* GnomeMeeting -- A Video-Conferencing application
 * Copyright (C) 2000-2001 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 *                         config.cpp  -  description
 *                         --------------------------
 *   begin                : Wed Feb 14 2001
 *   copyright            : (C) 2000-2002 by Damien Sandras 
 *   description          : This file contains most of gconf stuff.
 *                          All notifiers are here.
 *                          Callbacks that updates the gconf cache 
 *                          are in their file, except some generic one that
 *                          are in this file.
 *   Additional code      : Miguel Rodrguez Prez  <migrax@terra.es>
 *
 */


#ifndef _CONFIG_H
#define _CONFIG_H

#include <gnome.h>
#include <ptlib.h>

#include <gconf/gconf-client.h>

#include "common.h"


/* The functions */

/* DESCRIPTION  :  /
 * BEHAVIOR     :  This function inits all the notifiers
 *                 that GnomeMeeting uses.
 * PRE          :  /
 */
void gnomemeeting_init_gconf (GConfClient *);


/* DESCRIPTION  :  This callback is called when the user changes
 *                 an entry
 * BEHAVIOR     :  Updates the gconf cache, update the sensitivity of widgets for 
 *                 LDAP settings and gatekeeper.
 * PRE          :  data is the gconf key
 */
void entry_changed (GtkEditable  *, gpointer);


/* DESCRIPTION  :  This callback is called when the user changes               
 *                 the adjustment value                                        
 * BEHAVIOR     :  It updates the gconf cache                                  
 * PRE          :  /                                                           
 */
void adjustment_changed (GtkAdjustment  *, gpointer);


/* DESCRIPTION  :  This callback is called when the user changes               
 *                 the toggle value                                            
 * BEHAVIOR     :  It updates the gconf cache                                  
 * PRE          :  /                                                           
 */
void toggle_changed (GtkCheckButton  *, gpointer);


/* DESCRIPTION  :  This callback is called when the user changes               
 *                 the option menu value                                          
 * BEHAVIOR     :  It updates the gconf cache                                  
 * PRE          :  /                                                           
 */
void option_menu_changed (GtkWidget *, gpointer);

#endif
